classdef consolidated_controller_c < hgsetget
    
    properties
        Va
        last_Va
        Td
        last_Td
        last_n
        last_PD
        Tp
    end
    properties(Constant)
        n_max = 300/60;
        min_pitch = 0.5;
        max_pitch = 1.4;
        reminder_rounder = 3;
        fixed_pitch = 0.8;
    end
    
    methods 
        function obj = consolidated_controller_c(last_Va,last_Td) %Constructor
            obj.last_Va = last_Va;
            obj.last_Td = last_Td;
        end
        function [x0] = x0_creator(obj)                           %Used to create X_0 values
            x0 = [1,1,obj.Va,obj.Td];           
        end
        function [lb] = lb_creator(obj)                           %Creates the lower bound
            lb = [0.01,obj.min_pitch,obj.Va,obj.Td];
        end
        function [ub] = ub_creator(obj)                           %Creates the upper bound
            ub = [obj.n_max,obj.max_pitch,obj.Va,obj.Td];
        end
        function [n,PD] = solver(obj,Va,Td)                       %Finds the optimal combination of n and PD
            set(obj,'Va',Va); set(obj,'Td',Td);
             if(round(obj.Td,obj.reminder_rounder) == round(obj.last_Td,obj.reminder_rounder) && ... %%Checks if it can use the last values
                     round(obj.Va,obj.reminder_rounder) == round(obj.last_Va,obj.reminder_rounder))
                 n = obj.last_n;
                 PD = obj.last_PD;
             else
                x0 = obj.x0_creator; lb = obj.lb_creator; ub = obj.ub_creator;
               [x] = fmincon(@cost_func_consolidated,x0,[],[],[],[],lb,ub,@constraints_consolidated);
                n = x(1); PD = x(2);
                reminder(obj,x(1),x(2));
   
             end
        end
        function [n,PD] = solver_fixed_pitch(obj,Va,Td)          %Finds the right shaft speed to produce the desired thrust
            set(obj,'Va',Va); set(obj,'Td',Td);
             if(round(obj.Td,obj.reminder_rounder) == round(obj.last_Td,obj.reminder_rounder) && ...  %%Checks if it can use the last values
                     round(obj.Va,obj.reminder_rounder) == round(obj.last_Va,obj.reminder_rounder))
                 n = obj.last_n;
                 PD = obj.last_PD;
             else
                x0 = obj.x0_creator; lb = obj.lb_creator; lb(2) = obj.fixed_pitch; ub = obj.ub_creator; ub(2) = obj.fixed_pitch;
               [x] = fmincon(@cost_func_consolidated,x0,[],[],[],[],lb,ub,@constraints_consolidated);
                n = x(1);
                PD = x(2);
                reminder(obj,x(1),x(2));
             end
        end
        function[] = reminder(obj,n,PD)                             %% Stores the last values
            set(obj,'last_n',n);
            set(obj,'last_PD',PD);
            set(obj,'last_Va',obj.Va);
            set(obj,'last_Td',obj.Td);
        end
    end
end