classdef estimator < hgsetget
    properties
        last_n
        last_PD
        last_Qp
        last_Va_estimate
    end
    properties(Constant)
        min_J = 0;                      %Minimum value of the advance ratio
        max_J = inf;                    %Maximum value of the advance ratio
        D = 3;                          %Propeller diameter, used in advance ratio calculation
        confidence_factor = 0.2685;     %Used to see if the estimate will be valid
        surge_to_Va_ratio = 0.8;        %Used to create a rough estimate if the proper estimate will be unvalid
        reminder_rounder = 3;           %Used to rougly match old values with new in line 32
    end
    
    methods
        function obj = estimator(last_n,last_PD,last_Qp) %% Constructor 
            obj.last_n = last_n;
            obj.last_PD = last_PD;
            obj.last_Qp = last_Qp;
        end
        function [x0] = x0_creator(~,prop)
            x0 = [prop.n,prop.PD,prop.Qp,0];
        end
        function [lb] = lb_creator(obj,prop)
            lb = [prop.n,prop.PD,prop.Qp,obj.min_J];
        end
        function [ub] = ub_creator(obj,prop)
            ub = [prop.n,prop.PD,prop.Qp,obj.max_J];
        end
        function [Va,error] = advance_speed_estimate(obj,ship_surge,prop)
            if(     round(obj.last_n ,obj.reminder_rounder) == round(prop.n,obj.reminder_rounder) && ...
                    round(obj.last_PD,obj.reminder_rounder) == round(prop.PD,obj.reminder_rounder) && ...
                    round(obj.last_Qp,obj.reminder_rounder) == round(prop.Qp,obj.reminder_rounder))
                
                Va = obj.last_Va_estimate; %% If the last estimate was made with variables similar to the current ones, the last estimate is used
            else
                if prop.n > ship_surge * obj.confidence_factor %% Check to see if the estimate will be valid
                    x0 = obj.x0_creator(prop); lb = obj.lb_creator(prop); ub = obj.ub_creator(prop);
                   [x,error] = fmincon(@J_estimator,x0,[],[],[],[],lb,ub,@constraints_J);
                   clc;
                    Va = x(4);
                    obj.reminder(prop,Va);
                else  %%If the estimate will not be valid, a rough estimate is used instead
                    Va = ship_surge * obj.surge_to_Va_ratio;
                    obj.reminder(prop,Va);
                end
            end
        end
        function [] = reminder(obj,prop,Va)  %% Sets the last variables for the check on line 32
           set(obj,'last_n',prop.n);
           set(obj,'last_PD',prop.PD);
           set(obj,'last_Qp',prop.Qp);
           set(obj,'last_Va_estimate',Va);
        end
        function [J] = advance_ratio_estimate(obj,ship_surge,prop) %% Creates the advance ratio
            Va  = obj.advance_speed_estimate(ship_surge,prop);
            J = Va * (obj.n * obj.D)^-1;
        end
    end
end
