classdef ship < hgsetget
    
    properties
        surge
        sway
        yaw
        advance_speed
        vessel
        vesselABC
    end
    properties(Constant)
        Xuu = -10000;
        Yvv = 1;    
        Nvv = 1;      
    end
    methods
        function obj = ship(surge,sway,yaw,vessel,vesselABC)
            obj.surge = surge;
            obj.sway = sway;
            obj.yaw = yaw;
            obj.vessel = vessel;
            obj.vesselABC = vesselABC;
        end
        function [M] = mass_matrix(obj)
             M =   [obj.vessel.main.m + obj.vesselABC.MA(1,1), 0                                                           , 0;
                    0                                        , obj.vessel.main.m+obj.vesselABC.MA(2,2)                     , obj.vessel.main.m*obj.vessel.main.CG(1)+obj.vesselABC.MA(2,6);
                    0                                        , obj.vessel.main.m*obj.vessel.main.CG(1)+obj.vesselABC.MA(2,6), -obj.vesselABC.MRB(6,6)+obj.vesselABC.MA(6,6)];
        end
        function [N] = tot_damping(obj)
            %Sum of coriolis matrix C and damping matrix D 
            u = obj.surge; v = obj.sway; r = obj.yaw;             
                Ca = [0                                         , 0                     , - obj.vesselABC.MA(2,2)*v - obj.vesselABC.MA(2,6)*r; 
                      0                                         , 0                     , obj.vesselABC.MA(1,1)*u;
                      obj.vesselABC.MA(2,2)*v + obj.vesselABC.MA(2,6)*r , - obj.vesselABC.MA(1,1)*u , 0];
                D = [-obj.Xuu * u , 0           , 0;
                     0          , - obj.Yvv * v , - obj.Yvv * v;
                     0          , - obj.Nvv * v , - obj.Nvv * v];

                N = Ca + D;
        end
        function [dnu,damping] = eq_motion(obj,thrust_p)
            tau = zeros(3,1); tau(1) = thrust_p;
            nu = zeros(3,1); nu(1) = obj.surge; nu(2) = obj.sway; nu(3) = obj.yaw;
            M = obj.mass_matrix(); N = obj.tot_damping();
            
            dnu = M^-1*(tau - N*nu);
            damping = N*nu;
            
        end

        
    end
end