function [state_values,ship] = ship_system_fixed_pitch_step_estVA(t,states,ship,prop,speed_controller,motor,consolidated_ctrl,estimator,time_end)
    
    set(ship,  'surge',states(1));
    set(ship,  'sway' ,states(2));
    set(ship,  'yaw'  ,states(3));
    set(prop,  'n'    ,states(4));
    set(prop,  'PD'   ,states(6));

    
    
    %Equations of motion
        [dnu,damping] = ship.eq_motion(prop.thrust_p);
        
    %Advance speed
        real_Va = (1-0.2755)*ship.surge;
        estimated_Va = estimator.advance_speed_estimate(ship.surge,prop);
        
    %Speed Controller
        desired_speed_step = heaviside(t-5)*5;
        thrust_desired = speed_controller.speed_controll_rounded(dnu,desired_speed_step,ship);
    
    %Consolidated controller
        [nd,PDd] = consolidated_ctrl.solver_fixed_pitch(estimated_Va,thrust_desired);
    
    %Pitch dynamics
        dPD = prop.pitch_dynamics(PDd);
    
    %Propeller dynamics
        Qp = prop.torque_produced(real_Va);
        thrust_produced = prop.thrust_produced(real_Va);
    
    %Engine control
        error = round((nd-prop.n),3);
        error_int = states(5);
        Qm = motor.engine_torque(error,error_int); 
        if error ~= 0
            dn = (motor.Qm-prop.Qp)/prop.J_tot;
        else
            dn = 0;
        end

    %Outputs
        state_values = [dnu; dn; error; dPD;];

    %% Printouts
    
    clc;
    timeleft = time_end - t
    current_speed =  ship.surge
    desired_speed_step
    estimated_advance_speed = estimated_Va
    real_advance_speed = real_Va
end
    
