classdef speed_controller
    
    properties(Constant)
        rho = 1025;        %Density of water
        eta = 3;           %Tunable parameter
        C_d = 0.6995;      %Drag coefficient Cd = Hoerner(b.vessel.main.B,b.vessel.main.T)
        A = 103.68;        %Forward projected area
        phi = 1;           %Boundary layer thickness
        max_thrust = 5e5;  %Max thrust value
        round_decimal = 3; %Constant used in the rounded function
    end
    
    methods
        function [desired_thrust] = speed_controll(obj,dnu,desired_speed,ship) %% A function giving the desired thrust
           desired_thrust_temp = (ship.vessel.main.m - ship.vesselABC.MA(1,1)) * dnu(1) ...
               + 0.5 * obj.rho * obj.C_d * obj.A * ship.surge^2 ...
               - (ship.vessel.main.m + ship.vesselABC.MA(1,1)) * obj.eta * tanh((ship.surge - desired_speed)/obj.phi);
           if sign(desired_thrust_temp) == -1
               desired_thrust = desired_thrust_temp; 
           elseif desired_thrust_temp > obj.max_thrust
               desired_thrust = obj.max_thrust;
           else
               desired_thrust = desired_thrust_temp;
           end
        end
        function [desired_thrust] = speed_controll_rounded(obj,dnu,desired_speed,ship) %% A function returning a rounded value for the desired thrust
           desired_thrust_temp = (ship.vessel.main.m - ship.vesselABC.MA(1,1)) * dnu(1) ...
               + 0.5 * obj.rho * obj.C_d * obj.A * round(ship.surge,obj.round_decimal)^2 ...
               - (ship.vessel.main.m + ship.vesselABC.MA(1,1)) * obj.eta * tanh((round(ship.surge,obj.round_decimal) - round(desired_speed,obj.round_decimal))/obj.phi);
           if sign(desired_thrust_temp) == -1
               desired_thrust = 0; 
           elseif desired_thrust_temp > obj.max_thrust
               desired_thrust = obj.max_thrust;
           else
               desired_thrust = desired_thrust_temp;
           end
        end
    end
end
        